@@ -40,8 +40,8 @@ def find_checker(func: CallableT) -> Optional[CallableT]:
     return contract_checker
 
 
-def _kwargs_from_call(param_names: List[str], kwdefaults: Dict[str, Any], args: Tuple[Any, ...],
-                      kwargs: Dict[str, Any]) -> MutableMapping[str, Any]:
+def kwargs_from_call(param_names: List[str], kwdefaults: Dict[str, Any], args: Tuple[Any, ...],
+                     kwargs: Dict[str, Any]) -> MutableMapping[str, Any]:
     """
     Inspect the input values received at the wrapper for the actual function call.
 
@@ -52,7 +52,15 @@ def _kwargs_from_call(param_names: List[str], kwdefaults: Dict[str, Any], args:
     :return: resolved arguments as they would be passed to the function
     """
     # pylint: disable=too-many-arguments
-    resolved_kwargs = dict()  # type: MutableMapping[str, Any]
+
+    # (Marko Ristin, 2020-12-01)
+    # Insert _ARGS and _KWARGS preemptively even if they are not needed by any contract.
+    # This makes the code logic much simpler since we do not explicitly check if a contract would
+    # need them, though it might incur a subtle computational overhead
+    # (*e.g.*, when the contracts do not need them or don't use any argument at all).
+    # We need to have a concrete issue where profiling helps us determine if this is a real
+    # bottleneck or not and not optimize for no real benefit.
+    resolved_kwargs = {'_ARGS': args, '_KWARGS': kwargs}
 
     # Set the default argument values as condition parameters.
     for param_name, param_value in kwdefaults.items():
@@ -74,7 +82,7 @@ def _kwargs_from_call(param_names: List[str], kwdefaults: Dict[str, Any], args:
     return resolved_kwargs
 
 
-def _not_check(check: Any, contract: Contract) -> bool:
+def not_check(check: Any, contract: Contract) -> bool:
     """
     Negate the check value of a condition and capture missing boolyness (*e.g.*, when check is a numpy array).
 
@@ -95,14 +103,14 @@ def _not_check(check: Any, contract: Contract) -> bool:
         raise ValueError(''.join(msg_parts)) from err
 
 
-def _assert_precondition(contract: Contract, resolved_kwargs: Mapping[str, Any]) -> None:
+def select_condition_kwargs(contract: Contract, resolved_kwargs: Mapping[str, Any]) -> Mapping[str, Any]:
     """
-    Assert that the contract holds as a precondition.
+    Select the keyword arguments that are used by the contract.
 
     :param contract: contract to be verified
     :param resolved_kwargs:
         resolved keyword arguments of the call (including the default argument values of the decorated function)
-    :return:
+    :return: a subset of resolved_kwargs
     """
     # Check that all arguments to the condition function have been set.
     missing_args = [arg_name for arg_name in contract.mandatory_args if arg_name not in resolved_kwargs]
@@ -122,9 +130,23 @@ def _assert_precondition(contract: Contract, resolved_kwargs: Mapping[str, Any])
         for arg_name, value in resolved_kwargs.items() if arg_name in contract.condition_arg_set
     }
 
+    return condition_kwargs
+
+
+def _assert_precondition(contract: Contract, resolved_kwargs: Mapping[str, Any]) -> None:
+    """
+    Assert that the contract holds as a precondition.
+
+    :param contract: contract to be verified
+    :param resolved_kwargs:
+        resolved keyword arguments of the call (including the default argument values of the decorated function)
+    :return:
+    """
+    condition_kwargs = select_condition_kwargs(contract=contract, resolved_kwargs=resolved_kwargs)
+
     check = contract.condition(**condition_kwargs)
 
-    if _not_check(check=check, contract=contract):
+    if not_check(check=check, contract=contract):
         if contract.error is not None and (inspect.ismethod(contract.error) or inspect.isfunction(contract.error)):
             assert contract.error_arg_set is not None, "Expected error_arg_set non-None if contract.error a function."
             assert contract.error_args is not None, "Expected error_args non-None if contract.error a function."
@@ -163,7 +185,7 @@ def _assert_invariant(contract: Contract, instance: Any) -> None:
     else:
         check = contract.condition()
 
-    if _not_check(check=check, contract=contract):
+    if not_check(check=check, contract=contract):
         if contract.error is not None and (inspect.ismethod(contract.error) or inspect.isfunction(contract.error)):
             assert contract.error_arg_set is not None, "Expected error_arg_set non-None if contract.error a function."
             assert contract.error_args is not None, "Expected error_args non-None if contract.error a function."
@@ -224,7 +246,9 @@ def _assert_postcondition(contract: Contract, resolved_kwargs: Mapping[str, Any]
     both argument values captured in snapshots and actual argument values and the result of a function.
 
     :param contract: contract to be verified
-    :param resolved_kwargs: resolved keyword arguments (including the default values, ``result`` and ``OLD``)
+    :param resolved_kwargs:
+        resolved keyword arguments (including the default values, ``result``,``OLD``
+        ``_ARGS`` and ``_KWARGS``)
     :return:
     """
     assert 'result' in resolved_kwargs, \
@@ -253,7 +277,7 @@ def _assert_postcondition(contract: Contract, resolved_kwargs: Mapping[str, Any]
 
     check = contract.condition(**condition_kwargs)
 
-    if _not_check(check=check, contract=contract):
+    if not_check(check=check, contract=contract):
         if contract.error is not None and (inspect.ismethod(contract.error) or inspect.isfunction(contract.error)):
             assert contract.error_arg_set is not None, "Expected error_arg_set non-None if contract.error a function."
             assert contract.error_args is not None, "Expected error_args non-None if contract.error a function."
@@ -303,6 +327,18 @@ class _Old:
         return "a bunch of OLD values"
 
 
+def resolve_kwdefaults(sign: inspect.Signature) -> Dict[str, Any]:
+    """Resolve default values for the function arguments based on its signature."""
+    kwdefaults = dict()  # type: Dict[str, Any]
+
+    # Add to the defaults all the values that are needed by the contracts.
+    for param in sign.parameters.values():
+        if param.default != inspect.Parameter.empty:
+            kwdefaults[param.name] = param.default
+
+    return kwdefaults
+
+
 # This flag is used to avoid recursively checking contracts for the same function or instance while
 # contract checking is already in progress.
 #
@@ -324,15 +360,20 @@ def decorate_with_checker(func: CallableT) -> CallableT:
         "per function)."
 
     sign = inspect.signature(func)
+    if '_ARGS' in sign.parameters:
+        raise ValueError(
+            'The arguments of the function to be decorated with a contract checker include "_ARGS" which is '
+            'a reserved placeholder for positional arguments in the condition.')
+
+    if '_KWARGS' in sign.parameters:
+        raise ValueError(
+            'The arguments of the function to be decorated with a contract checker include "_KWARGS" which is '
+            'a reserved placeholder for keyword arguments in the condition.')
+
     param_names = list(sign.parameters.keys())
 
-    # Determine the default argument values.
-    kwdefaults = dict()  # type: Dict[str, Any]
-
-    # Add to the defaults all the values that are needed by the contracts.
-    for param in sign.parameters.values():
-        if param.default != inspect.Parameter.empty:
-            kwdefaults[param.name] = param.default
+    # Determine the default argument values
+    kwdefaults = resolve_kwdefaults(sign=sign)
 
     id_func = str(id(func))
 
@@ -340,6 +381,14 @@ def decorate_with_checker(func: CallableT) -> CallableT:
         """Wrap func by checking the preconditions and postconditions."""
         # pylint: disable=too-many-branches
 
+        if '_ARGS' in kwargs:
+            raise TypeError('The arguments of the function call include "_ARGS" which is '
+                            'a placeholder for positional arguments in a condition.')
+
+        if '_KWARGS' in kwargs:
+            raise TypeError('The arguments of the function call include "_KWARGS" which is '
+                            'a placeholder for keyword arguments in a condition.')
+
         # Use try-finally instead of ExitStack for performance.
         try:
             # If the wrapper is already checking the contracts for the wrapped function, avoid a recursive loop
@@ -353,8 +402,7 @@ def decorate_with_checker(func: CallableT) -> CallableT:
             snapshots = getattr(wrapper, "__postcondition_snapshots__")  # type: List[Snapshot]
             postconditions = getattr(wrapper, "__postconditions__")  # type: List[Contract]
 
-            resolved_kwargs = _kwargs_from_call(
-                param_names=param_names, kwdefaults=kwdefaults, args=args, kwargs=kwargs)
+            resolved_kwargs = kwargs_from_call(param_names=param_names, kwdefaults=kwdefaults, args=args, kwargs=kwargs)
 
             if postconditions:
                 if 'result' in resolved_kwargs:
@@ -429,13 +477,42 @@ def decorate_with_checker(func: CallableT) -> CallableT:
 
 def _find_self(param_names: List[str], args: Tuple[Any, ...], kwargs: Dict[str, Any]) -> Any:
     """Find the instance of ``self`` in the arguments."""
-    instance_i = param_names.index("self")
-    if instance_i < len(args):
-        instance = args[instance_i]
-    else:
-        instance = kwargs["self"]
+    instance_i = None
+    try:
+        instance_i = param_names.index("self")
+    except ValueError:
+        pass
 
-    return instance
+    if instance_i is not None:
+        return args[instance_i]
+
+    return kwargs["self"]
+
+
+def _decorate_new_with_invariants(new_func: CallableT) -> CallableT:
+    """
+    Decorate the ``__new__`` of a class s.t. the invariants are checked on the result.
+
+    This is necessary for optimized classes such as ``namedtuple`` which use ``object.__init__``
+    as constructor and do not expect a wrapping around the constructor.
+    """
+    if _already_decorated_with_invariants(func=new_func):
+        return new_func
+
+    def wrapper(*args, **kwargs):  # type: ignore
+        """Pass the arguments to __new__ and check invariants on the result."""
+        instance = new_func(*args, **kwargs)
+
+        for contract in instance.__class__.__invariants__:
+            _assert_invariant(contract=contract, instance=instance)
+
+        return instance
+
+    functools.update_wrapper(wrapper=wrapper, wrapped=new_func)
+
+    setattr(wrapper, "__is_invariant_check__", True)
+
+    return wrapper  # type: ignore
 
 
 def _decorate_with_invariants(func: CallableT, is_init: bool) -> CallableT:
@@ -458,15 +535,21 @@ def _decorate_with_invariants(func: CallableT, is_init: bool) -> CallableT:
 
         def wrapper(*args, **kwargs):  # type: ignore
             """Wrap __init__ method of a class by checking the invariants *after* the invocation."""
-            result = func(*args, **kwargs)
-            instance = _find_self(param_names=param_names, args=args, kwargs=kwargs)
-            assert instance is not None, "Expected to find `self` in the parameters, but found none."
+            try:
+                instance = _find_self(param_names=param_names, args=args, kwargs=kwargs)
+            except KeyError as err:
+                raise KeyError(("The parameter 'self' could not be found in the call to function {!r}: "
+                                "the param names were {!r}, the args were {!r} and kwargs were {!r}").format(
+                                    func, param_names, args, kwargs)) from err
 
+            # We need to disable the invariants check during the constructor.
             id_instance = str(id(instance))
             setattr(_IN_PROGRESS, id_instance, True)
 
             # ExitStack is not used here due to performance.
             try:
+                result = func(*args, **kwargs)
+
                 for contract in instance.__class__.__invariants__:
                     _assert_invariant(contract=contract, instance=instance)
 
@@ -479,12 +562,15 @@ def _decorate_with_invariants(func: CallableT, is_init: bool) -> CallableT:
 
         def wrapper(*args, **kwargs):  # type: ignore
             """Wrap a function of a class by checking the invariants *before* and *after* the invocation."""
-            #
+            try:
+                instance = _find_self(param_names=param_names, args=args, kwargs=kwargs)
+            except KeyError as err:
+                raise KeyError(("The parameter 'self' could not be found in the call to function {!r}: "
+                                "the param names were {!r}, the args were {!r} and kwargs were {!r}").format(
+                                    func, param_names, args, kwargs)) from err
+
             # The following dunder indicates whether another invariant is currently being checked. If so,
             # we need to suspend any further invariant check to avoid endless recursion.
-            instance = _find_self(param_names=param_names, args=args, kwargs=kwargs)
-            assert instance is not None, "Expected to find `self` in the parameters, but found none."
-
             id_instance = str(id(instance))
             if not hasattr(_IN_PROGRESS, id_instance):
                 setattr(_IN_PROGRESS, id_instance, True)
@@ -534,6 +620,8 @@ def _already_decorated_with_invariants(func: CallableT) -> bool:
 
 def add_invariant_checks(cls: type) -> None:
     """Decorate each of the class functions with invariant checks if not already decorated."""
+    # pylint: disable=too-many-branches
+
     # Candidates for the decoration as list of (name, dir() value)
     init_name_func = None  # type: Optional[Tuple[str, Callable[..., None]]]
     names_funcs = []  # type: List[Tuple[str, Callable[..., None]]]
@@ -541,10 +629,11 @@ def add_invariant_checks(cls: type) -> None:
 
     # Filter out entries in the directory which are certainly not candidates for decoration.
     for name, value in [(name, getattr(cls, name)) for name in dir(cls)]:
+        # __new__ is a special class method (though not marked properly with @classmethod!).
         # We need to ignore __repr__ to prevent endless loops when generating error messages.
         # __getattribute__, __setattr__ and __delattr__ are too invasive and alter the state of the instance.
         # Hence we don't consider them "public".
-        if name in ["__repr__", "__getattribute__", "__setattr__", "__delattr__"]:
+        if name in ["__new__", "__repr__", "__getattribute__", "__setattr__", "__delattr__"]:
             continue
 
         if name == "__init__":
@@ -577,8 +666,16 @@ def add_invariant_checks(cls: type) -> None:
 
     if init_name_func:
         name, func = init_name_func
-        wrapper = _decorate_with_invariants(func=func, is_init=True)
-        setattr(cls, name, wrapper)
+
+        # We have to distinguish this special case which is used by named
+        # tuples and possibly other optimized data structures.
+        # In those cases, we have to wrap __new__ instead of __init__.
+        if func == object.__init__ and hasattr(cls, "__new__"):
+            new_func = getattr(cls, "__new__")
+            setattr(cls, "__new__", _decorate_new_with_invariants(new_func))
+        else:
+            wrapper = _decorate_with_invariants(func=func, is_init=True)
+            setattr(cls, name, wrapper)
 
     for name, func in names_funcs:
         wrapper = _decorate_with_invariants(func=func, is_init=False)
