@@ -4,7 +4,9 @@ import ast
 import builtins
 import functools
 import platform
-from typing import Any, Mapping, Dict, List, Optional, Union, Tuple, Set, Callable  # pylint: disable=unused-import
+import sys
+from typing import Any, Mapping, Dict, List, Optional, Union, Tuple, Set, Callable, \
+    cast  # pylint: disable=unused-import
 
 
 class Placeholder:
@@ -36,8 +38,11 @@ class Visitor(ast.NodeVisitor):
 
         :param variable_lookup: list of lookup tables to look-up the values of the variables, sorted by precedence
         """
-        # Resolve precedence of variable lookup
+        # _name_to_value maps the variable names to variable values.
+        # This is important for Load contexts as well as Store contexts in, e.g., named expressions.
         self._name_to_value = dict()  # type: Dict[str, Any]
+
+        # Resolve precedence of variable lookups
         for lookup in variable_lookup:
             for name, value in lookup.items():
                 if name not in self._name_to_value:
@@ -46,28 +51,82 @@ class Visitor(ast.NodeVisitor):
         # value assigned to each visited node
         self.recomputed_values = dict()  # type: Dict[ast.AST, Any]
 
-    def visit_Num(self, node: ast.Num) -> Union[int, float]:
-        """Recompute the value as the number at the node."""
-        result = node.n
+    # pylint: disable=no-member
+    if sys.version_info < (3, 8):
 
-        self.recomputed_values[node] = result
+        def visit_Num(self, node: ast.Num) -> Union[int, float]:
+            """Recompute the value as the number at the node."""
+            result = node.n
 
-        assert isinstance(result, (int, float))
-        return result
+            self.recomputed_values[node] = result
 
-    def visit_Str(self, node: ast.Str) -> str:
-        """Recompute the value as the string at the node."""
-        result = node.s
+            assert isinstance(result, (int, float))
+            return result
 
-        self.recomputed_values[node] = result
-        return result
+        def visit_Str(self, node: ast.Str) -> str:
+            """Recompute the value as the string at the node."""
+            result = node.s
 
-    def visit_Bytes(self, node: ast.Bytes) -> bytes:
-        """Recompute the value as the bytes at the node."""
-        result = node.s
+            self.recomputed_values[node] = result
+            return result
 
-        self.recomputed_values[node] = result
-        return node.s
+        def visit_Bytes(self, node: ast.Bytes) -> bytes:
+            """Recompute the value as the bytes at the node."""
+            result = node.s
+
+            self.recomputed_values[node] = result
+            return node.s
+
+        def visit_NameConstant(self, node: ast.NameConstant) -> Any:
+            """Forward the node value as a result."""
+            self.recomputed_values[node] = node.value
+            return node.value
+    else:
+
+        def visit_Constant(self, node: ast.Constant) -> Any:
+            """Forward the node value as a result."""
+            self.recomputed_values[node] = node.value
+            return node.value
+
+    if sys.version_info >= (3, 6):
+
+        def visit_FormattedValue(self, node: ast.FormattedValue) -> Any:
+            """Format the node value."""
+            fmt = ['{']
+            # See https://docs.python.org/3/library/ast.html#ast.FormattedValue for these
+            # constants
+            if node.conversion == -1:
+                pass
+            elif node.conversion == 115:
+                fmt.append('!s')
+            elif node.conversion == 114:
+                fmt.append('!r')
+            elif node.conversion == 97:
+                fmt.append('!a')
+            else:
+                raise NotImplementedError("Unhandled conversion of a formatted value node {!r}: {}".format(
+                    node, node.conversion))
+
+            if node.format_spec is not None:
+                fmt.append(":")
+
+                # The following assert serves only documentation purposes so that the code is easier to follow.
+                assert isinstance(node.format_spec, ast.JoinedStr)
+                fmt.append(self.visit(node.format_spec))
+
+            fmt.append('}')
+
+            recomputed_value = self.visit(node.value)
+            return ''.join(fmt).format(recomputed_value)
+
+        def visit_JoinedStr(self, node: ast.JoinedStr) -> Any:
+            """Visit the values and concatenate them."""
+            joined_str = ''.join(self.visit(value_node) for value_node in node.values)
+
+            self.recomputed_values[node] = joined_str
+            return joined_str
+
+    # pylint: enable=no-member
 
     def visit_List(self, node: ast.List) -> List[Any]:
         """Visit the elements and assemble the results into a list."""
@@ -108,11 +167,6 @@ class Visitor(ast.NodeVisitor):
         self.recomputed_values[node] = recomputed_dict
         return recomputed_dict
 
-    def visit_NameConstant(self, node: ast.NameConstant) -> Any:
-        """Forward the node value as a result."""
-        self.recomputed_values[node] = node.value
-        return node.value
-
     def visit_Name(self, node: ast.Name) -> Any:
         """Load the variable by looking it up in the variable look-up and in the built-ins."""
         if not isinstance(node.ctx, ast.Load):
@@ -302,6 +356,24 @@ class Visitor(ast.NodeVisitor):
         self.recomputed_values[node] = result
         return result
 
+    if sys.version_info >= (3, 8):
+        # pylint: disable=no-member
+        def visit_NamedExpr(self, node: ast.NamedExpr) -> Any:
+            """Visit the node's ``value`` and assign it to both this node and the target."""
+            value = self.visit(node=node.value)
+            self.recomputed_values[node] = value
+
+            # This assignment is needed to make mypy happy.
+            target = cast(ast.Name, node.target)
+
+            if not isinstance(target.ctx, ast.Store):
+                raise NotImplementedError(
+                    "Expected Store context in the target of a named expression, but got: {}".format(target.ctx))
+
+            self._name_to_value[target.id] = value
+
+            return value
+
     def visit_Index(self, node: ast.Index) -> Any:
         """Visit the node's ``value``."""
         result = self.visit(node=node.value)
